from src.mkslug import check_for_errors, generate


class TestMakeSlug:
    def test_check_for_errors_regular_sentence(self):
        """
        Test check_for_errors function works successfully when a regular sentence provided.
        """

        error, error_text = check_for_errors("Test SLUG vaLUEs 1")

        assert not error
        assert error_text is None

    def test_check_for_errors_empty_sentence(self):
        """
        Test check_for_errors function raising an error when no sentence provided.
        """

        error, error_text = check_for_errors("")

        assert error is True
        assert error_text is not None

    def test_check_for_errors_special_characters_in_sentence(self):
        """
        Test check_for_errors function raising an error when special characters in a sentence provided.
        """

        error, error_text = check_for_errors("%One slug ^* value  $  ")

        assert error is True
        assert error_text is not None

    def test_generate_regular_sentence(self):
        """
        Test generate function works successfully when a regular sentence provided.
        """

        slug = generate("Test SLUG vaLUEs 1")

        assert slug == "test-slug-values-1"

    def test_generate_outer_chars_are_blank(self):
        """
        Test generate function works successfully when the outer characters are blank provided.
        """

        slug = generate("    Test SLUG vaLUEs 2  ")

        assert slug == "test-slug-values-2"



import re


def check_for_errors(sentence: str) -> tuple[bool, str]:
    """
    Check for errors on the sentence.

    Args:
        sentence: The sentence to check.

    Returns:
        error bool and error text message.
    """

    error = True
    error_text = None

    while error:

        if not sentence:
            error_text = "The sentence must be populated. Please try again."
            break

        pattern = "[^a-zA-Z0-9\s-]"
        found = re.findall(pattern, sentence)
        if found:
            regex_error_text = ""
            found_len = len(found)

            for i in range(found_len):
                regex_error_text += found[i]

            error_text = f"Found one or more special characters in the sentence ({regex_error_text}). Please try again without any special characters."
            break

        error = False

    return error, error_text


def generate(sentence: str) -> str:
    """
    Generate a slug from a sentence.

    Args:
        sentence: The sentence to check.

    Returns:
        generated slug value.
    """

    slug = sentence.strip()
    slug = slug.lower()
    slug = slug.replace(" ", "-")

    return slug



import mkslug


def main() -> None:

    print("Make a Slug from Sentence You Provide.")

    error = True

    while error:
        sentence = input("Please enter one sentence and press the return/enter key: ")
        error, error_text = mkslug.check_for_errors(sentence)
        if error:
            print(error_text)

    slug = mkslug.generate(sentence)
    print("The generated slug is:")
    print(f"{slug}")


if __name__ == "__main__":
    main()
